from openfisca_uk.data.datasets import (
    RawFRS,
    RawLCFS,
    RawWAS,
    RawSPI,
    FRS,
    SPI,
    ExtendedFRS,
    CalibratedFRS,
    EnhancedFRS,
    LCFS,
    WAS,
    SynthFRS,
    DATASETS,
)

EnhancedFRS = EnhancedFRS
